import logging

from sqlalchemy.orm.attributes import ScalarObjectAttributeImpl

from rest_framework.decorators import api_view, renderer_classes
from rest_framework.response import Response
from rest_framework.renderers import JSONRenderer
from rest_framework.views import exception_handler as _exception_handler

from gargantext.models import ProjectNode
from gargantext.core.task import schedule

from .exceptions import base_exception_handler
from .celery_app import app

logger = logging.getLogger(__name__)


def exception_handler(exc, context):
    response = base_exception_handler(exc, context)

    if response is not None:
        code = getattr(response.data.get('detail'), 'code', None)
        if code:
            response.data['code'] = code
        response.data['status_code'] = response.status_code

    return response


# https://bitbucket.org/zzzeek/sqlalchemy/issues/3976/built-in-way-to-convert-an-orm-object-ie
from sqlalchemy import inspect
def orm_is_value(a):
    return hasattr(a, 'key') and \
        not isinstance(getattr(a, 'impl'), ScalarObjectAttributeImpl)
def orm_to_dict(obj):
    attrs = inspect(type(obj)).all_orm_descriptors
    return {a.key: getattr(obj, a.key) for a in attrs if orm_is_value(a)}


@api_view(['GET'])
@renderer_classes((JSONRenderer,))
def projects_view(request):
    r = list(map(orm_to_dict, request.db.query(ProjectNode).all()))

    return Response({'results': r})


def task_view(task, *args, **kwargs):
    @api_view(['GET'])
    @renderer_classes((JSONRenderer,))
    def view(request, **params):
        logging.getLogger('gargantext').info('From view: %r', request.user)
        r = schedule(task, user=request.user, args=args, kwargs=kwargs)
        return Response({
            "task": task.__name__,
            "id": r.id,
        })

    return view


@api_view(['GET'])
@renderer_classes((JSONRenderer,))
def task_state(request, task_id):
    r = app.AsyncResult(task_id)
    result = r.result

    json_value = (bool, str, int, float, dict, list, tuple, type(None))

    return Response({
        "id": r.id,
        "failed": r.failed(),
        "finished": r.ready(),
        "successful": r.successful(),
        "state": r.state,
        "result": result if isinstance(result, json_value) else repr(r.result),
    })



